# Part 0: 시작하며 - 12주 완성, AI 서비스 개발 마스터 과정

안녕하세요! "AI 서비스 개발 12주 완성" 과정에 오신 것을 환영합니다.

AI 서비스를 만드는 것은 마치 **'스스로 생각하고 행동하는 기계'**를 조립하는 것과 같습니다. 처음에는 작은 부품(프로그래밍 기초)을 다루는 법부터 배우고, 점차 엔진(AI 모델)을 만들어 올리고, 마지막에는 멋진 차체(API 서버)를 씌워 실제 세상(인터넷)을 달리게 하는 것과 같죠.

이 과정은 프로그래밍 기초부터 시작해, AI 모델을 활용한 고성능 API 서버를 구축하고, 최종적으로는 여러분만의 아이디어를 실현하는 캡스톤 프로젝트까지 완료하는 것을 목표로 합니다. 12주 동안 막연하게 느껴졌던 'AI 서비스 개발'의 모든 과정을 구체적인 코드로 구현하며, 현업에 바로 투입될 수 있는 실무 역량을 갖추게 될 것입니다.

## 🏁 최종 목표: 나만의 AI 서비스 포트폴리오 완성

단순한 예제 완성을 넘어, 우리는 이 과정을 통해 **"실제 운영 가능한 수준의 AI 기반 API 서버"**를 구축하고, 이를 응용한 **"캡스톤 프로젝트"**를 완성하여 자신만의 포트폴리오를 만들게 됩니다.

이는 마치 **'레시피만 따라 요리 하나를 만드는 것'**과 **'손님들이 언제든 찾아와 주문할 수 있는 식당 주방을 설계하는 것'**의 차이와 같습니다. 우리는 후자를 목표로 합니다. 언제든 요청에 응답하고, 안정적으로 서비스를 제공하는 '프로의 시스템'을 만드는 경험을 하게 될 것입니다.

이 과정을 통해 여러분은 다음과 같은 결과물을 얻게 됩니다.

- **탄탄한 Python 프로그래밍 역량**: 기초 문법부터 객체 지향 설계까지 깊이 있는 이해
- **고성능 AI API 서버**: FastAPI를 기반으로 한 비동기 API 서버 구축 및 운영 능력
- **실전 머신러닝 모델링 경험**: 데이터 전처리, 모델 학습, 평가, 배포까지 전 과정 경험
- **최신 DevOps 지식**: Docker를 이용한 컨테이너화 및 CI/CD 자동화 파이프라인 개념 이해
- **나만의 포트폴리오**: 아이디어를 실제 서비스로 구현한 캡스톤 프로젝트 결과물

## 🙋‍♂️ 대상 독자

이 과정은 다음과 같은 분들을 위해 체계적으로 설계되었습니다.

- 프로그래밍은 처음이지만 AI 서비스를 직접 만들어보고 싶은 **입문자**
- Python 기본 문법은 알지만, 실제 프로젝트 경험이 부족한 **학생 또는 주니어 개발자**
- AI 모델은 개발했지만, 이를 어떻게 서비스로 만들어야 할지 막막한 **연구자 및 데이터 분석가**

## 🗺️ 12주 학습 로드맵

우리는 12주 동안 다음의 로드맵에 따라 체계적으로 학습을 진행합니다. 이 로드맵은 AI 서비스 개발이라는 보물섬을 찾아가는 **'친절한 안내 지도'**입니다. 각 주차별 목표를 따라 한 걸음씩 나아가다 보면, 어느새 유능한 개발자로 성장한 자신을 발견하게 될 것입니다.

```mermaid
graph TD
    subgraph "1부: 파이썬 기초 다지기 (1-4주)"
        W1["<a href='part_1_ai_development_environment.md'><b>1주차</b>: 과정 소개 및<br/>개발 환경 설정</a>"]
        W2["<a href='part_2_python_core_syntax.md'><b>2주차</b>: 파이썬 핵심 문법</a>"]
        W3["<a href='part_3_python_collections.md'><b>3주차</b>: 파이썬 컬렉션 심화</a>"]
        W4["<a href='part_4_object_oriented_programming.md'><b>4.주차</b>: 객체 지향 프로그래밍 (OOP)</a>"]
    end

    subgraph "2부: AI 모델 개발 및 API 서버 구축 (5-9주)"
        W5["<a href='part_5_ai_core_libraries.md'><b>5주차</b>: 핵심 라이브러리<br/>(NumPy & Pandas)</a>"]
        W6["<a href='part_6_machine_learning.md'><b>6주차</b>: 머신러닝 완전 정복<br/>(회귀, 분류, 군집, 튜닝)</a>"]
        W7["<a href='part_7_deep_learning.md'><b>7주차</b>: 딥러닝 첫걸음<br/>(DNN, CNN, RNN, Transformer)</a>"]
        W8["<a href='part_8_model_serving_with_fastapi.md'><b>8주차</b>: FastAPI를 이용한<br/>모델 서빙(API)</a>"]
        W9["<a href='part_9_production_ready_api.md'><b>9주차</b>: API 서버 고도화 및<br/>Docker 패키징</a>"]
    end

    subgraph "3부: 심화 학습 및 캡스톤 프로젝트 (10-12주)"
        W10["<a href='part_10_expert_path.md'><b>10주차</b>: 전문가 과정 소개 (MLOps)</a>"]
        W11["<a href='part_11_capstone_project_guide.md'><b>11주차</b>: 캡스톤 프로젝트 (Phase 1)</a>"]
        W12["<a href='part_11_capstone_project_guide.md'><b>12주차</b>: 캡스톤 프로젝트 (Phase 2) 및<br/>마무리</a>"]
    end

    W1 --> W2 --> W3 --> W4 --> W5 --> W6 --> W7 --> W8 --> W9 --> W10 --> W11 --> W12
```

## 🚀 AI의 미래: 에이전트 워크플로우 (The Future of AI: Agentic Workflows)

최근 AI 분야의 가장 뜨거운 화두는 바로 **'AI 에이전트'**입니다. 스탠포드 대학의 앤드류 응(Andrew Ng) 교수는 AI의 미래가 에이전트에 달려있다고 강조합니다.

> **💡 AI 에이전트란?**
>
> 단순히 주어진 질문에 한 번 답하고 끝나는 것이 아니라, 목표를 달성하기 위해 **스스로 계획을 세우고, 도구를 사용하며, 스스로 피드백하고 수정하는 과정을 반복**하는 지능형 시스템을 말합니다.
>
> 이는 마치 우리가 복잡한 문제를 해결하기 위해 여러 번 생각하고, 자료를 찾아보고, 계획을 수정하는 과정과 유사합니다. 이러한 **에이전틱 워크플로우(Agentic Workflow)**는 기존 모델보다 훨씬 더 뛰어난 문제 해결 능력을 보여줍니다.
>
> (참고: [Agents: The Future of AI, Explained by Dr. Andrew Ng](https://medium.com/@honeyricky1m3/agents-the-future-of-ai-explained-by-dr-andrew-ng-9be4675a6079))

본 과정의 후반부에서는 이러한 최신 트렌드를 맛보고, 여러분이 만든 AI 모델을 어떻게 더 똑똑한 '에이전트'로 발전시킬 수 있을지에 대한 아이디어를 얻게 될 것입니다.

## ❓ 자주 묻는 질문 (Q&A)

**Q1: 정말 프로그래밍을 하나도 몰라도 따라갈 수 있나요?**
> A1: 네, 가능합니다. 본 과정의 1부(1-4주차)는 비전공자 및 입문자를 위해 파이썬 기초를 매우 상세하게 다룹니다. 열정과 꾸준함만 있다면 누구나 과정을 완주할 수 있습니다.

**Q2: 과정에서 사용하는 주요 기술은 무엇인가요?**
> A2: <a href="glossary.md#python">Python</a>, <a href="glossary.md#numpy">NumPy</a>, <a href="glossary.md#pandas">Pandas</a>, <a href="glossary.md#scikit-learn">Scikit-learn</a>, <a href="glossary.md#fastapi">FastAPI</a>, <a href="glossary.md#docker">Docker</a> 등 현재 AI 서비스 개발 현업에서 가장 널리 사용되는 기술 스택을 중심으로 학습합니다.

**Q3: 개인 노트북 사양이 많이 좋아야 하나요?**
> A3: 기본적인 코딩은 일반적인 노트북으로 충분합니다. 모델 학습 등 높은 사양이 필요한 경우, 무료로 사용할 수 있는 <a href="glossary.md#google-colab">Google Colab</a>을 활용하는 방법을 안내해 드리므로 걱정하지 않으셔도 됩니다.

**Q4: 캡스톤 프로젝트는 어떻게 진행되나요?**
> A4: 11-12주차에 진행되며, 수강생들이 직접 주제를 선정하고 기획, 개발, 발표까지 진행합니다. 이 과정에서 멘토링을 통해 프로젝트가 올바른 방향으로 나아갈 수 있도록 적극적으로 지원합니다. 자세한 내용은 <a href="part_11_capstone_project_guide.md">캡스톤 프로젝트 가이드</a>를 참고하세요.

---

자, 이제 12주간의 여정을 시작할 준비가 되셨나요?
다음 시간에는 본격적인 개발을 위해 여러분의 컴퓨터에 개발 환경을 설정하는 방법을 알아보겠습니다.

**➡️ 다음 시간: [Part 1: AI 개발 환경 준비](part_1_ai_development_environment.md)** 